package net.wanji.utc.service.control;

import net.wanji.utc.entity.JsonViewObject;
import net.wanji.utc.service.rpc.RpcService;
import net.wanji.utc.vo.*;

/**
 * 控制指令 service
 *
 * @author wuxiaokai
 * @date 2022/11/15 14:02:36
 */
public interface ControlCommandService extends RpcService {

    /**
     * 方案下发
     */
    JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception;

    /**
     * 计划下发
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    JsonViewObject planSend(PlanSendVO planSendVO) throws Exception;

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     */
    JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception;

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception;

    /**
     * 步进控制
     *
     * @param code    信号机编号
     * @param command 1 开始步进 0 取消步进
     * @param stepNum 0 顺序步进 n 跳过n个相位
     * @return
     * @throws Exception 异常
     */
    JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception;

    /**
     * 设置信号机控制模式
     *
     * @param code        信号机编号
     * @param command     指令 1 开 0 关
     * @param commandType 命令类型
     * @return
     * @throws Exception 异常
     */
    JsonViewObject setSignalControl(String code, Integer command, Integer commandType) throws Exception;

    /**
     * 恢复时间表
     *
     * @param code 信号机编号
     * @return
     */
    JsonViewObject recoverSchedule(String code) throws Exception;

    /**
     * 相位配时下发
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception;

    /**
     * 相位差下发
     */
    JsonViewObject phaseDiffSend();

}
