package net.wanji.utc.service.control.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.common.typeenum.DateStyle;
import net.wanji.utc.entity.JsonViewObject;
import net.wanji.utc.mapper.*;
import net.wanji.utc.po.*;
import net.wanji.utc.service.control.ControlCommandStrategyService;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.util.StringUtils;
import net.wanji.utc.vo.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/12 9:53
 */
@Service
@RequiredArgsConstructor
public class ControlCommandStrategyServiceImpl implements ControlCommandStrategyService {

    @Resource(name = "hkControlCommandService")
    ControlCommandService hkControlCommandService;
    @Resource(name = "wanJiControlCommandService")
    ControlCommandService wanJiControlCommandService;
    private final CrossInfoMapper crossInfoMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;


    /**
     * 通过路口编号获取厂商编码
     *
     * @param crossId
     * @return
     */
    public String getManufacturerCodeByCrossId(String crossId) throws Exception{
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        if (Objects.isNull(crossInfoPO)) {
            throw new Exception("当前路口编号不存在!");
        }
        String signalId = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商代码
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        return manufacturerInfoPO.getCode();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public JsonViewObject schemeSendStrategy(SchemeSendVO schemeSendVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(schemeSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.schemeSend(schemeSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.schemeSend(schemeSendVO);
        }
        schemeSendDataInsert(schemeSendVO);
        return jsonViewObject;
    }

    private void schemeSendDataInsert(SchemeSendVO schemeSendVO) {
        // 入库操作
        List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
        schemeSendVO.getPatternList().forEach(pattern -> {
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            crossSchemePO.setSchemeNo(pattern.getPatternNo());
            crossSchemePO.setName(pattern.getPatternName());
            crossSchemePO.setCrossId(schemeSendVO.getCrossCode());
            crossSchemePO.setCycle(Integer.parseInt(pattern.getCycle()));
            crossSchemePO.setCoordPhase(Integer.parseInt(pattern.getCoordPhase()));
            crossSchemePO.setOffset(Integer.parseInt(pattern.getOffset()));
            crossSchemePO.setSource(2);
            crossSchemePO.setIsDeleted(0);
            crossSchemePOList.add(crossSchemePO);

            pattern.getRings().forEach(ring -> {
                List<CrossPhasePO> crossPhasePOList = new ArrayList<>();
                ring.getPhaseList().forEach(phase -> {
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    crossPhasePO.setPhaseNo(phase.getPhaseNo());
                    crossPhasePO.setName(phase.getPhaseName());
                    crossPhasePO.setSort(Integer.parseInt(phase.getSort()));
                    crossPhasePO.setCrossId(schemeSendVO.getCrossCode());
                    crossPhasePO.setPlanId(Integer.parseInt(pattern.getPatternNo()));
                    crossPhasePO.setRingNo(Integer.parseInt(ring.getRingNo()));
                    crossPhasePO.setControlMode(Integer.parseInt(phase.getControlMode()));
                    crossPhasePO.setPhaseTime(Integer.parseInt(phase.getPhaseTime()));
                    crossPhasePO.setGreenTime(Integer.parseInt(phase.getGreenTime()));
                    crossPhasePO.setGreenFlashTime(Integer.parseInt(phase.getGreenFlashTime()));
                    crossPhasePO.setPedFlashTime(Integer.parseInt(phase.getPedFlashTime()));
                    crossPhasePO.setYellowTime(Integer.parseInt(phase.getYellowTime()));
                    crossPhasePO.setRedTime(Integer.parseInt(phase.getRedTime()));
                    crossPhasePO.setMinGreenTime(Integer.parseInt(phase.getMinGreenTime()));
                    crossPhasePO.setMaxGreenTime(Integer.parseInt(phase.getMaxGreenTime()));
                    crossPhasePOList.add(crossPhasePO);
                });
                // 入库路口相位表
                crossPhaseMapper.insertBatch(crossPhasePOList);
            });

        });
        // 入库信号方案表
        crossSchemeMapper.insertBatch(crossSchemePOList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public JsonViewObject planSendStrategy(PlanSendVO planSendVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(planSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.planSend(planSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.planSend(planSendVO);
        }
        planSendDataInsert(planSendVO);
        return jsonViewObject;
    }

    private void planSendDataInsert(PlanSendVO planSendVO) {
        List<CrossPlanPO> crossPlanPOList = new ArrayList<>();
        planSendVO.getPlanList().forEach(plan -> {
            CrossPlanPO crossPlanPO = new CrossPlanPO();
            crossPlanPO.setPlanNo(plan.getPlanNo());
            crossPlanPO.setName(plan.getPlanDescribe());
            crossPlanPO.setCrossId(planSendVO.getCrossCode());
            crossPlanPOList.add(crossPlanPO);
            List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
            plan.getSectionList().forEach(section -> {
                CrossSectionPO crossSectionPO = new CrossSectionPO();
                crossSectionPO.setSectionNo(section.getSectionNo());
                crossSectionPO.setStartTime(section.getBeginTime());
                crossSectionPO.setEndTime(section.getEndTime());
                crossSectionPO.setCrossId(planSendVO.getCrossCode());
                crossSectionPO.setPlanId(Integer.parseInt(plan.getPlanNo()));
                crossSectionPO.setControlMode(Integer.parseInt(section.getControlMode()));
                crossSectionPO.setScemeId(Integer.parseInt(section.getPatternNo()));
                crossSectionPOList.add(crossSectionPO);
            });
            // 入库路口时段表
            crossSectionMapper.insertBatch(crossSectionPOList);
        });
        // 入库路口计划表
        crossPlanMapper.insertBatch(crossPlanPOList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public JsonViewObject scheduleSendStrategy(ScheduleSendVO scheduleSendVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(scheduleSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.scheduleSend(scheduleSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.scheduleSend(scheduleSendVO);
        }
        scheduleSendDataInsert(scheduleSendVO);
        return jsonViewObject;
    }

    private void scheduleSendDataInsert(ScheduleSendVO scheduleSendVO) {
        // 1、入库操作
        SimpleDateFormat sdf = new SimpleDateFormat(DateStyle.YYYY_MM_DD.getValue());
        ArrayList<CrossSchedulesPO> crossSchedulesPOS = new ArrayList<>();
        scheduleSendVO.getSchedules().forEach(schedule -> {
            schedule.getWeeks().forEach(week -> {
                CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                if (week.getWeekNum() >= 6) {
                    schedulesPO.setName("周末");
                } else {
                    schedulesPO.setName("工作日");
                }
                schedulesPO.setCrossId(scheduleSendVO.getCrossCode());
                schedulesPO.setPlanId(week.getPlanNo());
                schedulesPO.setWeek(week.getWeekNum());
                schedulesPO.setSpecialDate(null);
                crossSchedulesPOS.add(schedulesPO);
            });
            schedule.getSpecialDays().forEach(specialDay -> {
                CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                schedulesPO.setName("特殊日期");
                schedulesPO.setCrossId(scheduleSendVO.getCrossCode());
                schedulesPO.setPlanId(specialDay.getPlanNo());
                schedulesPO.setWeek(null);
                try {
                    schedulesPO.setSpecialDate(sdf.parse(specialDay.getDateStr()));
                } catch (ParseException e) {
                    schedulesPO.setSpecialDate(null);
                }
                crossSchedulesPOS.add(schedulesPO);
            });
        });
        // 入库时间表
        crossSchedulesMapper.insertBatch(crossSchedulesPOS);
    }

    @Override
    public JsonViewObject lockControlStrategy(ControlCommandVO commandVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(commandVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.lockControl(commandVO);
        } else {
            jsonViewObject = wanJiControlCommandService.lockControl(commandVO);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepControlStrategy(String crossId, Integer command, Integer stepNum) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.stepControl(crossId, command, stepNum);
        } else {
            jsonViewObject = wanJiControlCommandService.stepControl(crossId, command, stepNum);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject setSignalControlStrategy(String crossId, Integer command, Integer commandType) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.setSignalControl(crossId, command, commandType);
        } else {
            jsonViewObject = wanJiControlCommandService.setSignalControl(crossId, command, commandType);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject recoverScheduleStrategy(String crossId) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(crossId)) {
            return jsonViewObject.fail("路口编号不能为空");
        }
        String manufacturerIdCode = getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.recoverSchedule(crossId);
        } else {
            jsonViewObject = wanJiControlCommandService.recoverSchedule(crossId);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSendStrategy(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = getManufacturerCodeByCrossId(phaseTimingSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.phaseTimingSend(phaseTimingSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.phaseTimingSend(phaseTimingSendVO);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseDiffSendStrategy() {
        return null;
    }
}
