package net.wanji.utc.service.staticinfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.HkPlanSectionControlTypeEnum;
import net.wanji.utc.mapper.CrossPlanMapper;
import net.wanji.utc.mapper.CrossSchemeMapper;
import net.wanji.utc.mapper.CrossSectionMapper;
import net.wanji.utc.po.CrossPlanPO;
import net.wanji.utc.po.CrossSchemePO;
import net.wanji.utc.po.CrossSectionPO;
import net.wanji.utc.service.staticinfo.HkPlanSectionService;
import net.wanji.utc.util.PathUtil;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Map;

/**
 * @author Kent HAN
 * @date 2022/11/18 16:34
 */
@Service
@RequiredArgsConstructor
public class HkPlanSectionServiceImpl implements HkPlanSectionService {
    private final ArtemisConfig artemisConfig;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossSectionMapper crossSectionMapper;

    @Override
    public void planSection(String crossCode, Integer planNo, String crossId) throws Exception {
        // 请求3.1.6接口，获取信号机计划
        Map<String, String> path = PathUtil.getPathMapByApiCode("uploadPlan");
        JSONObject dataEle = new JSONObject();
        dataEle.put("crossCode", crossCode);
        dataEle.put("planNo", planNo);
        ArrayList<JSONObject> data = new ArrayList<>();
        data.add(dataEle);
        JSONObject requestContent = new JSONObject();
        requestContent.put("data", data);
        String body = requestContent.toJSONString();
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                null, "application/json", null);
        JSONObject responseObj = JSON.parseObject(responseStr);
        if(Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray responsedata = responseObj.getJSONArray("data");
            for (int n = 0; n < responsedata.size(); n++) {
                JSONObject responseDataEle = responsedata.getJSONObject(n);
                JSONArray planList = responseDataEle.getJSONArray("planList");
                for (int i = 0; i < planList.size(); i++) {
                    CrossPlanPO crossPlanPO = new CrossPlanPO();
                    JSONObject plan = planList.getJSONObject(i);
                    String responsePlanNo = plan.getString("planNo");
                    crossPlanPO.setPlanNo(responsePlanNo);
                    String name = plan.getString("planName");
                    crossPlanPO.setName(name);
                    crossPlanPO.setCrossId(crossId);
                    // 写入数据库
                    crossPlanMapper.deleteOne(crossId, Integer.parseInt(responsePlanNo));
                    crossPlanMapper.insertOne(crossPlanPO);
                    Integer crossPlanPOId = crossPlanPO.getId();
                    // 构造时段数据
                    JSONArray sectionList = plan.getJSONArray("sectionList");
                    for (int j = 0; j < sectionList.size(); j++) {
                        JSONObject section = sectionList.getJSONObject(j);
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        String timeSecNo = section.getString("timeSecNo");
                        crossSectionPO.setSectionNo(timeSecNo);
                        String beginTime = section.getString("beginTime");
                        crossSectionPO.setStartTime(beginTime);
                        // 计算结束时间
                        if (j < sectionList.size() - 1) {
                            JSONObject nextSection = sectionList.getJSONObject(j + 1);
                            String endTime = nextSection.getString("beginTime");
                            crossSectionPO.setEndTime(endTime);
                        } else {
                            crossSectionPO.setEndTime("24:00");
                        }
                        crossSectionPO.setCrossId(crossId);
                        crossSectionPO.setPlanId(crossPlanPOId);
                        Integer controlType = section.getInteger("controlType");
                        controlType = HkPlanSectionControlTypeEnum.getCodeByHkCode(controlType);
                        crossSectionPO.setControlMode(controlType);
                        // 获取方案ID
                        Integer schemeNo = section.getInteger("patternNo");
                        CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, schemeNo);
                        Integer schemePOId = crossSchemePO.getId();
                        crossSectionPO.setScemeId(schemePOId);
                        // 写入数据库
                        crossSectionMapper.deleteOne(crossId, Integer.parseInt(timeSecNo));
                        crossSectionMapper.insertOne(crossSectionPO);
                    }
                }
            }
        }
    }
}
