package net.wanji.utc.service.staticinfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.*;
import net.wanji.utc.entity.cache.PhaseCache;
import net.wanji.utc.mapper.*;
import net.wanji.utc.po.*;
import net.wanji.utc.service.staticinfo.HkSchemePhaseLightsService;
import net.wanji.utc.util.PathUtil;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 方案数据-方案信息、相位信息
 *
 * @author Kent HAN
 * @date 2022/11/16 15:27
 */
@Service
@RequiredArgsConstructor
public class HkSchemePhaseLightsServiceImpl implements HkSchemePhaseLightsService {
    private final ArtemisConfig artemisConfig;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;

    @Override
    public void hkSchemePhaseLights(String crossId, String crossCode) throws Exception {
        // 匹配用缓存
        Map<Integer, PhaseCache> phaseNoMap = new HashMap<>();
        Map<Integer, PhaseCache> laneNoMap = new HashMap<>();
        // 请求3.1.11接口，获取并缓存全量相位数据
        handlePhaseData(crossCode, phaseNoMap, laneNoMap);
        // 请求3.1.3接口，获取并处理方案列表
        Map<String, String> path = PathUtil.getPathMapByApiCode("uploadPatternCodeList");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray data = object.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                JSONObject obj =  data.getJSONObject(i);
                Integer patternNo = obj.getInteger("patternNo");
                String patternName = obj.getString("patternName");
                handleSchemeData(crossId, crossCode, patternNo, patternName, phaseNoMap);
            }
        }
        // 请求3.1.2接口，获取并处理灯组数据
        handleLightsData(crossId, crossCode, laneNoMap);
    }

    private void handleLightsData(String crossId, String crossCode, Map<Integer, PhaseCache> laneNoMap)
            throws Exception {
        Map<String, String> path = PathUtil.getPathMapByApiCode("getCrossInfo");
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = new ArrayList<>();
        JSONObject requestContent = new JSONObject();
        // 海康固定入参字段名
        requestContent.put("pageNo", 1);
        requestContent.put("pageSize", 9999);
        JSONObject searchObj = new JSONObject();
        requestContent.put("searchObj", searchObj);
        ArrayList<String> codeLst = new ArrayList<>();
        codeLst.add(crossCode);
        searchObj.put("codeLst", codeLst);
        String body = requestContent.toJSONString();
        // 请求参数
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                null, "application/json", null);
        JSONObject responseObj = JSON.parseObject(responseStr);
        if(Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
            JSONObject data = responseObj.getJSONObject("data");
            JSONArray list = data.getJSONArray("list");
            for(int i = 0; i < list.size(); i++){
                JSONObject crossObj = list.getJSONObject(i);
                JSONArray laneEles = crossObj.getJSONArray("laneEles");
                for (int j = 0; j < laneEles.size(); j++) {
                    JSONObject laneObj = laneEles.getJSONObject(j);
                    CrossLightsPO crossLightsPO = new CrossLightsPO();
                    String lightsNo = laneObj.getString("channelNo");
                    crossLightsPO.setLightsNo(lightsNo);
                    String laneNo = laneObj.getString("laneNo");
                    crossLightsPO.setName(laneNo);
                    Integer lampGroupType = laneObj.getInteger("lampGroupType");
                    Integer type = HkLightsTypeEnum.getCodeByHkCode(lampGroupType);
                    crossLightsPO.setType(type);
                    Integer direction = laneObj.getInteger("direction");
                    Integer dir = HkLightsDirEnum.getCodeByHkCode(direction);
                    crossLightsPO.setDir(dir);
                    Integer turn = laneObj.getInteger("turn");
                    turn = HkLightsTurnEnum.getCodeByHkCode(turn);
                    crossLightsPO.setTurn(turn);
                    crossLightsPO.setCrossId(crossId);
                    if (type != null && type == 3) { // 3行人灯
                        crossLightsPO.setInOutType(3); // 海康都是进出口
                    }
                    // 插入数据返回自增ID
                    crossLightsMapper.deleteOne(crossId, laneNo);
                    crossLightsMapper.insertOne(crossLightsPO);
                    Integer crossLightsPOId = crossLightsPO.getId();
                    // 匹配缓存信息，构造相位灯组关系对象列表
                    PhaseCache phaseCache = laneNoMap.get(Integer.parseInt(laneNo));
                    if (phaseCache != null) {
                        Integer phaseNo = phaseCache.getPhaseNo();
                        List<Integer> ids = crossPhaseMapper.selectIdsByPhaseNo(phaseNo);
                        if (ids != null) {
                            for (Integer id : ids) {
                                CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                                crossPhaseLightsPO.setLightsId(crossLightsPOId);
                                crossPhaseLightsPO.setPhaseId(id);
                                crossPhaseLightsPO.setCrossId(crossId);
                                crossPhaseLightsPOList.add(crossPhaseLightsPO);
                            }
                        }
                    }
                }
            }
        }
        // 更新相位-灯组关系
        crossPhaseLightsMapper.deleteByCrossId(crossId);
        crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
    }

    private void handlePhaseData(String crossCode, Map<Integer, PhaseCache> phaseNoMap,
                                 Map<Integer, PhaseCache> laneNoMap) throws Exception {
        Map<String, String> path = PathUtil.getPathMapByApiCode("getPhaseRlatLanes");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONObject data = object.getJSONObject("data");
            JSONArray list = data.getJSONArray("phaseList");
            for (int i = 0; i < list.size(); i++) {
                PhaseCache phaseCache = new PhaseCache();
                JSONObject phase = list.getJSONObject(i);
                // 从接口返回对象中获取数据
                Integer phaseNo = phase.getInteger("phaseNo");
                phaseCache.setPhaseNo(phaseNo);
                String desc = phase.getString("desc");
                phaseCache.setDesc(desc);
                Integer yellow = phase.getInteger("yellow");
                phaseCache.setYellow(yellow);
                Integer allred = phase.getInteger("allred");
                phaseCache.setAllred(allred);
                Integer greenFlash = phase.getInteger("greenFlash");
                phaseCache.setGreenFlash(greenFlash);
                Integer redFlash = phase.getInteger("redFlash");
                phaseCache.setRedFlash(redFlash);
                Integer minGreen = phase.getInteger("minGreen");
                phaseCache.setMinGreen(minGreen);
                Integer maxGreen = phase.getInteger("maxGreen");
                phaseCache.setMaxGreen(maxGreen);
                phaseNoMap.put(phaseNo, phaseCache);
                // 构造车道Map
                JSONArray lanes = phase.getJSONArray("lanes");
                for (int j = 0; j < lanes.size(); j++) {
                    JSONObject lane = lanes.getJSONObject(j);
                    Integer laneNo = lane.getInteger("laneNo");
                    laneNoMap.put(laneNo, phaseCache);
                }
            }
        }
    }

    private void handleSchemeData(String crossId, String crossCode, Integer patternNo, String patternName,
                                  Map<Integer, PhaseCache> phaseNoMap) throws Exception {

        // 请求3.1.4接口，获取并处理方案详情
        Map<String, String> path = PathUtil.getPathMapByApiCode("getCrossPlanDetail");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        queryMap.put("patternNo", patternNo.toString());
        String Result = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject object = JSON.parseObject(Result);
        if(Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
            JSONArray data = object.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                JSONObject scheme =  data.getJSONObject(i);
                Integer cycle = scheme.getInteger("cycle");
                Integer coordPhase = scheme.getInteger("coordPhase");
                Integer offset = scheme.getInteger("offset");
                // 构造数据库记录
                CrossSchemePO crossSchemePO = new CrossSchemePO();
                crossSchemePO.setSchemeNo(patternNo.toString());
                crossSchemePO.setName(patternName);
                crossSchemePO.setCrossId(crossId);
                crossSchemePO.setCycle(cycle);
                crossSchemePO.setCoordPhase(coordPhase);
                crossSchemePO.setOffset(offset);
                crossSchemePO.setSource(2);
                crossSchemePO.setIsDeleted(0);
                // 插入方案数据返回自增ID
                CrossSchemePO crossSchemePO1 = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, patternNo);
                if (crossSchemePO1 != null) {
                    Integer schemeId = crossSchemePO1.getId();
                    crossPhaseMapper.deleteBatch(crossId, schemeId.toString());
                    crossSchemeMapper.deleteOne(crossId, patternNo.toString());
                }
                crossSchemeMapper.insertOne(crossSchemePO);
                Integer schemePOId = crossSchemePO.getId();
                // 生成相位数据
                JSONArray phases = scheme.getJSONArray("phaseInfos");
                for (int j = 0; j < phases.size(); j++) {
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    JSONObject phase = phases.getJSONObject(j);
                    Integer phaseNo = phase.getInteger("phaseNo");
                    PhaseCache phaseCache = phaseNoMap.get(phaseNo);

                    crossPhasePO.setPhaseNo(phaseNo.toString());
                    String phaseName = phase.getString("phaseName");
                    crossPhasePO.setName(phaseName);
                    Integer phaseSequence = phase.getInteger("phaseSequence");
                    crossPhasePO.setSort(phaseSequence);
                    crossPhasePO.setCrossId(crossId);
                    crossPhasePO.setPlanId(schemePOId);
                    Integer ringNo = phase.getInteger("ringNo");
                    crossPhasePO.setRingNo(ringNo);
                    crossPhasePO.setControlMode(1); // 海康均为定周期
                    Integer phaseLength = phase.getInteger("phaseLength");

                    crossPhasePO.setPhaseTime(phaseLength);
                    Integer greenFlash = phaseCache.getGreenFlash();
                    Integer yellow = phaseCache.getYellow();
                    Integer allred = phaseCache.getAllred();
                    int green = phaseLength - greenFlash - yellow - allred;
                    crossPhasePO.setGreenTime(green);
                    crossPhasePO.setGreenFlashTime(greenFlash);
                    crossPhasePO.setYellowTime(yellow);
                    crossPhasePO.setRedTime(allred);
                    crossPhasePO.setMinGreenTime(phaseCache.getMinGreen());
                    crossPhasePO.setMaxGreenTime(phaseCache.getMaxGreen());
                    // 插入数据
                    crossPhaseMapper.insertOne(crossPhasePO);
                }
            }
        }
    }
}
